@@ -43,6 +43,10 @@
 #include <QTimer>
 #include <QDir>
 #include <QSessionManager>
+#include <QThread>
+#include <QDBusConnection>
+#include <QDBusInterface>
+#include <QDBusReply>
 
 #include <shell/core.h>
 #include <shell/mainwindow.h>
@@ -55,6 +59,7 @@
 #include <shell/session.h>
 #include <interfaces/ilauncher.h>
 #include <interfaces/iproject.h>
+#include <interfaces/foregroundlock.h>
 
 #include "kdevideextension.h"
 
@@ -76,10 +81,19 @@ public:
         KApplication::saveState(sm);
     }
 
+    virtual bool notify(QObject* receiver, QEvent* event);
 };
 
+bool KDevelopApplication::notify(QObject* receiver, QEvent* event)
+{
+    KDevelop::ForegroundLock lock(QThread::currentThread() == qApp->thread());
+    return KApplication::notify(receiver, event);
+}
+
 int main( int argc, char *argv[] )
 {
+    KDevelop::ForegroundLock lock;
+    
     static const char description[] = I18N_NOOP( "The KDevelop Integrated Development Environment" );
     KAboutData aboutData( "kdevelop", 0, ki18n( "KDevelop" ),
                           i18n("%1", QString(VERSION) ).toUtf8(), ki18n(description), KAboutData::License_GPL,
@@ -169,19 +183,36 @@ int main( int argc, char *argv[] )
 
     KCmdLineArgs::init( argc, argv, &aboutData );
     KCmdLineOptions options;
-    options.add("n").add("cs <name>", ki18n("Create new session with given name."));
-    options.add("s").add("open-session <session>", ki18n("Session to load. You can pass either hash or the name of the session." ));
-    options.add("l").add("sessions", ki18n( "List available sessions and quit" ));
-    options.add("p").add("project <project>", ki18n( "Url to project to load" ));
-    options.add("d").add("debug <debugger>", ki18n( "Start debugger, for example gdb. The binary that should be debugged must follow - including arguments." ));
+    options.add("n")
+           .add("new-session <name>", ki18n("Open KDevelop with a new session using the given name."));
+    options.add("s")
+           .add("open-session <session>", ki18n("Open KDevelop with the given session.\n"
+                                           "You can pass either hash or the name of the session." ));
+    options.add("ps").add("pick-session", ki18n("Shows all available sessions and lets you select one to open." ));
+    options.add("l")
+           .add("list-sessions", ki18n( "List available sessions and quit." ));
+    options.add("p")
+           .add("project <project>", ki18n( "Open KDevelop and load the given project." ));
+    options.add("d")
+           .add("debug <debugger>",
+                ki18n( "Start debugging an application in KDevelop with the given debugger.\n"
+                       "The binary that should be debugged must follow - including arguments.\n"
+                       "Example: kdevelop --debug gdb myapp --foo bar"));
 
     options.add("+files", ki18n( "Files to load" ));
 
+    options.add(":", ki18n("Deprecated options:"));
+    options.add("sessions", ki18n( "Same as -l / --list-sessions" ));
+    options.add("cs <name>", ki18n( "Same as -n / --new-session" ));
+
     KCmdLineArgs::addCmdLineOptions( options );
     KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
-    KApplication app;
+    KDevelopApplication app;
 
-    if(args->isSet("sessions"))
+    // The session-controller needs to arguments to eventually pass them to newly opened sessions
+    KDevelop::SessionController::setArguments(argc, argv);
+
+    if(args->isSet("sessions") || args->isSet("list-sessions"))
     {
         QTextStream qout(stdout);
         qout << endl << ki18n("Available sessions (use '-s HASH' or '-s NAME' to open a specific one):").toString() << endl << endl;
@@ -203,6 +234,13 @@ int main( int argc, char *argv[] )
 
     // if empty, restart kdevelop with last active session, see SessionController::defaultSessionId
     QString session;
+    
+    if(args->isSet("ps"))
+    {
+        session = KDevelop::SessionController::showSessionChooserDialog();
+        if(session.isEmpty())
+            return 1;
+    }
 
     if ( args->isSet("debug") ) {
         if ( debugArgs.isEmpty() ) {
@@ -215,9 +253,9 @@ int main( int argc, char *argv[] )
             binary = binary.right(binary.lastIndexOf('/'));
         }
         session = i18n("Debug")+" "+binary;
-    } else if ( args->isSet("cs") )
+    } else if ( args->isSet("cs") || args->isSet("new-session") )
     {
-        session = args->getOption("cs");
+        session = args->isSet("cs") ? args->getOption("cs") : args->getOption("new-session");
         foreach(const KDevelop::SessionInfo& si, KDevelop::SessionController::availableSessionInfo())
         {
             if ( session == si.name ) {
@@ -244,6 +282,55 @@ int main( int argc, char *argv[] )
         }
     }
 
+    QString sessionId = session;
+    foreach(const KDevelop::SessionInfo& si, KDevelop::SessionController::availableSessionInfo()) {
+        if ( si.name == session ) {
+            sessionId = si.uuid.toString();
+            break;
+        }
+    }
+
+    forever {
+        KDevelop::SessionController::LockSessionState state = KDevelop::SessionController::tryLockSession(sessionId);
+        if(!state) {
+            QDBusInterface interface(QString("org.kdevelop.kdevelop-%1").arg(state.holderPid),
+                                     "/kdevelop/MainWindow", "org.kdevelop.MainWindow",
+                                     QDBusConnection::sessionBus());
+            if (interface.isValid()) {
+                QDBusReply<void> reply = interface.call("ensureVisible");
+                if (reply.isValid()) {
+                    qDebug() << i18n("made running kdevelop instance (PID: %1) visible", state.holderPid);
+                    return 0;
+                }
+            }
+
+            QString errmsg = i18n("<p>Failed to lock the session <em>%1</em>, "
+                                  "already locked by %2 (PID %4) on %3.<br>"
+                                  "The given application did not respond to a DBUS call, "
+                                  "it may have crashed or is hanging.</p>"
+                                  "<p>Do you want to remove the lock file and force a new KDevelop instance?<br/>"
+                                  "<strong>Beware:</strong> Only do this if you are sure there is no running"
+                                  " KDevelop process using this session.</p>",
+                                  session, state.holderApp, state.holderHostname, state.holderPid );
+
+            KGuiItem overwrite = KStandardGuiItem::cont();
+            overwrite.setText(i18n("remove lock file"));
+            KGuiItem cancel = KStandardGuiItem::quit();
+            int ret = KMessageBox::warningYesNo(0, errmsg, i18n("Failed to Lock Session %1", session),
+                                                overwrite, cancel, QString() );
+            if (ret == KMessageBox::Yes) {
+                if (!QFile::remove(state.lockFile)) {
+                    KMessageBox::error(0, i18n("Failed to remove lock file %1.", state.lockFile));
+                    return 1;
+                }
+            } else {
+                return 1;
+            }
+        } else {
+            break;
+        }
+    }
+
     KDevIDEExtension::init();
 
     KSplashScreen* splash = 0;
@@ -377,5 +464,6 @@ int main( int argc, char *argv[] )
         args->clear();
     }
 
+    lock.unlock();
     return app.exec();
 }
