@@ -107,18 +107,6 @@ DebugSession::~DebugSession()
     delete commandQueue_;
 }
 
-void DebugSession::emitShowStepInSource(const QString& file, int line, const QString& address)
-{
-    kDebug() << file << line << address;
-    emit gdbShowStepInSource(file, line, address);
-    if (!file.isEmpty()) {
-        // Debugger counts lines from 1
-        emit showStepInSource(KUrl::fromPath(file), line-1);
-    } else {
-        emit clearExecutionPoint();
-    }
-}
-
 KDevelop::IDebugSession::DebuggerState DebugSession::state() const {
     return m_sessionState;
 }
@@ -750,12 +738,15 @@ void DebugSession::slotProgramStopped(const GDBMI::ResultRecord& r)
         
         if (r.hasField("frame")) {
             const GDBMI::Value& frame = r["frame"];
-            if (frame.hasField("fullname")
-                && frame.hasField("line")
-                && frame.hasField("addr")) {
-                emitShowStepInSource(frame["fullname"].literal(),
-                     frame["line"].literal().toInt(),
-                     frame["addr"].literal());
+            QString file, line, addr;
+            
+            if (frame.hasField("fullname")) file = frame["fullname"].literal();;
+            if (frame.hasField("line"))     line = frame["line"].literal();
+            if (frame.hasField("addr"))     addr = frame["addr"].literal();
+            
+            if (!file.isEmpty()) {
+                // gdb counts lines from 1 and we don't
+                setCurrentPosition(KUrl::fromLocalFile(file), line.toInt()-1, addr);
 
                 raiseEvent(program_state_changed);
                 state_reload_needed = false;
@@ -925,6 +916,9 @@ bool DebugSession::startDebugger(KDevelop::ILaunchConfiguration* cfg)
     
     queueCmd(new GDBCommand(GDBMI::EnablePrettyPrinting));
 
+    queueCmd(new GDBCommand(GDBMI::GdbSet, "charset UTF-8"));
+    queueCmd(new GDBCommand(GDBMI::GdbSet, "print sevenbit-strings off"));
+
     QString fileName = KStandardDirs::locate("data", "kdevgdb/printers/gdbinit");
     if (!fileName.isEmpty()) {
         queueCmd(new GDBCommand(GDBMI::NonMI, "source " + fileName));
@@ -968,26 +962,32 @@ bool DebugSession::startProgram(KDevelop::ILaunchConfiguration* cfg)
         }
     }
 
+    IExecutePlugin* iface = KDevelop::ICore::self()->pluginController()->pluginForExtension("org.kdevelop.IExecutePlugin")->extension<IExecutePlugin>();
+    Q_ASSERT(iface);
 
     // Configuration values
-    bool    config_breakOnLoadingLibrary_ = grp.readEntry( GDBDebugger::breakOnLibLoadEntry, false );
     bool    config_forceBPSet_ = grp.readEntry( GDBDebugger::allowForcedBPEntry, true );
     bool    config_displayStaticMembers_ = grp.readEntry( GDBDebugger::staticMembersEntry, false );
     bool    config_asmDemangle_ = grp.readEntry( GDBDebugger::demangleNamesEntry, true );
-    //bool    config_dbgTerminal_ = grp.readEntry( GDBDebugger::separateTerminalEntry, false );
-    bool    config_dbgTerminal_ = false;
     KUrl config_dbgShell_ = grp.readEntry( GDBDebugger::debuggerShellEntry, KUrl() );
     KUrl config_configGdbScript_ = grp.readEntry( GDBDebugger::remoteGdbConfigEntry, KUrl() );
     KUrl config_runShellScript_ = grp.readEntry( GDBDebugger::remoteGdbShellEntry, KUrl() );
     KUrl config_runGdbScript_ = grp.readEntry( GDBDebugger::remoteGdbRunEntry, KUrl() );
     int config_outputRadix_ = 10;
+    
+    bool config_useExternalTerminal = iface->useTerminal( cfg );
+    QString config_externalTerminal = iface->terminal( cfg );
+    if (!config_externalTerminal.isEmpty()) {
+        // the external terminal cmd contains additional arguments, just get the terminal name
+        config_externalTerminal = KShell::splitArgs(config_externalTerminal).first();
+    }
 
     // Need to set up a new TTY for each run...
     if (tty_)
         delete tty_;
 
-    tty_ = new STTY(config_dbgTerminal_);//, Settings::terminalEmulatorName( *KGlobal::config() ));
-    if (!config_dbgTerminal_)
+    tty_ = new STTY(config_useExternalTerminal, config_externalTerminal);
+    if (!config_useExternalTerminal)
     {
         connect( tty_, SIGNAL(OutOutput(const QByteArray&)), SIGNAL(ttyStdout(const QByteArray&)) );
         connect( tty_, SIGNAL(ErrOutput(const QByteArray&)), SIGNAL(ttyStderr(const QByteArray&)) );
@@ -1009,10 +1009,6 @@ bool DebugSession::startProgram(KDevelop::ILaunchConfiguration* cfg)
 
     queueCmd(new GDBCommand(GDBMI::InferiorTtySet, tty));
 
-
-    IExecutePlugin* iface = KDevelop::ICore::self()->pluginController()->pluginForExtension("org.kdevelop.IExecutePlugin")->extension<IExecutePlugin>();
-    Q_ASSERT(iface);
-
     // Only dummy err here, actual erros have been checked already in the job and we don't get here if there were any
     QString err;
     QString executable = iface->executable(cfg, err).toLocalFile();
