@@ -61,7 +61,6 @@ static QString remoteGdbShellEntry = "Remote GDB Shell Script";
 static QString remoteGdbRunEntry = "Remote GDB Run Script";
 static QString staticMembersEntry = "Display Static Members";
 static QString demangleNamesEntry = "Display Demangle Names";
-static QString breakOnLibLoadEntry = "Try Setting Breakpoints On Loading Libraries";
 //static QString separateTerminalEntry = "Separate Terminal For Application IO";
 static QString allowForcedBPEntry = "Allow Forced Breakpoint Set";
 
@@ -114,7 +113,6 @@ public Q_SLOTS:
 
 private Q_SLOTS:
     void slotDebuggerAbnormalExit();
-    void emitShowStepInSource(const QString& file, int line, const QString& address);
 
 private:
     void _gdbStateChanged(DBGStateFlags oldState, DBGStateFlags newState);
@@ -288,7 +286,6 @@ Q_SIGNALS:
     void gdbInternalCommandStdout (const QString& output);
     void gdbUserCommandStdout (const QString& output);
     void gdbStateChanged(DBGStateFlags oldState, DBGStateFlags newState);
-    void gdbShowStepInSource(const QString &fileName, int line, const QString &address);
 
     void debuggerAbnormalExit();
 
