@@ -128,20 +128,20 @@ void AStylePreferences::updateWidgets()
     m_enableWidgetSignals = false;
     //indent
     if(m_formatter->option("Fill").toString() == "Tabs") {
+        chkConvertTabs->setEnabled(false);
+        chkConvertTabs->setChecked(false);
         if(m_formatter->option("FillForce").toBool()) {
             cbIndentType->setCurrentIndex(INDENT_TABSFORCE);
-            chkConvertTabs->setEnabled(false);
         } else {
             cbIndentType->setCurrentIndex(INDENT_TABS);
-            chkConvertTabs->setEnabled(false);
         }
     } else {
         cbIndentType->setCurrentIndex(INDENT_SPACES);
         chkConvertTabs->setEnabled(true);
+        chkConvertTabs->setChecked(m_formatter->option("FillForce").toBool());
     }
     inpNuberSpaces->setValue(m_formatter->option("FillCount").toInt());
     chkFillEmptyLines->setChecked(m_formatter->option("FillEmptyLines").toBool());
-    chkConvertTabs->setChecked(m_formatter->option("FillForce").toBool());
 
     // indent objects
     setItemChecked(INDENT_BLOCK, m_formatter->option("IndentBlocks").toBool());
@@ -231,23 +231,26 @@ void AStylePreferences::indentChanged()
 {
     if(!m_enableWidgetSignals)
         return;
-    
+
+
     switch(cbIndentType->currentIndex()) {
         case INDENT_TABS:
+            m_formatter->setTabSpaceConversionMode( false );
             m_formatter->setTabIndentation(inpNuberSpaces->value(), false);
             chkConvertTabs->setEnabled(false);
             break;
         case INDENT_TABSFORCE:
+            m_formatter->setTabSpaceConversionMode( false );
             m_formatter->setTabIndentation(inpNuberSpaces->value(), true);
             chkConvertTabs->setEnabled(false);
             break;
         case INDENT_SPACES:
             m_formatter->setSpaceIndentation(inpNuberSpaces->value());
             chkConvertTabs->setEnabled(true);
+            m_formatter->setTabSpaceConversionMode( chkConvertTabs->isEnabled() & chkConvertTabs->isChecked() );
             break;
     }
-    
-    m_formatter->setTabSpaceConversionMode( chkConvertTabs->isEnabled() & chkConvertTabs->isChecked() );
+
     m_formatter->setFillEmptyLines( chkFillEmptyLines->isChecked() );
 
     updatePreviewText();
