@@ -57,6 +57,7 @@
 #include <templateparameterdeclaration.h>
 #include <language/duchain/classdeclaration.h>
 #include "qpropertydeclaration.h"
+#include "model.h"
 
 // #define ifDebug(x) x
 
@@ -160,7 +161,7 @@ bool removePrefixWord(QString& expression, QString word) {
   return false;
 }
 
-CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context, const QString& text, const QString& followingText, const KDevelop::SimpleCursor& position, int depth, const QStringList& knownArgumentExpressions, int line )
+CodeCompletionContext::CodeCompletionContext(KDevelop::DUContextPointer context, const QString& text, const QString& followingText, const KDevelop::CursorInRevision& position, int depth, const QStringList& knownArgumentExpressions, int line )
   : KDevelop::CodeCompletionContext(context, text, position, depth),
     m_memberAccessOperation(NoMemberAccess),
     m_knownArgumentExpressions(knownArgumentExpressions),
@@ -828,9 +829,10 @@ void CodeCompletionContext::processFunctionCallAccess() {
       return;
     }
 
-    helper.setOperator(OverloadResolver::Parameter(m_expressionResult.type.abstractType(), m_expressionResult.isLValue()), m_operator);
+    helper.setOperator(OverloadResolver::Parameter(m_expressionResult.type.abstractType(), m_expressionResult.isLValue()));
 
     m_functionName = "operator"+m_operator;
+    
   } else {
     ///Simply take all the declarations that were found by the expression-parser
 
@@ -842,14 +844,17 @@ void CodeCompletionContext::processFunctionCallAccess() {
         m_functionName = decl->identifier().toString();
     }
   }
-
+  
+  if( m_contextType == BinaryOperatorFunctionCall || m_expression == m_functionName )
+    helper.setFunctionNameForADL( QualifiedIdentifier(m_functionName) );
+  
   OverloadResolver::ParameterList knownParameters;
   foreach( const ExpressionEvaluationResult &result, m_knownArgumentTypes )
     knownParameters.parameters << OverloadResolver::Parameter( result.type.abstractType(), result.isLValue() );
 
   helper.setKnownParameters(knownParameters);
 
-  m_functions = helper.resolve(true);
+  m_functions = helper.resolveToList(true);
 
   if(m_contextType == BinaryOperatorFunctionCall) {
     //Filter away all global binary operators that do not have the first argument matched
@@ -1141,7 +1146,7 @@ void getOverridable(DUContext* base, DUContext* current, QMap< QPair<IndexedType
         if(classFun->isDestructor())
           key.second = IndexedString("~" + key.second.str());
       }
-      if(!overridable.contains(key) && base->findLocalDeclarations(KDevelop::Identifier(key.second), SimpleCursor::invalid(), 0, key.first.abstractType(), KDevelop::DUContext::OnlyFunctions).isEmpty())
+      if(!overridable.contains(key) && base->findLocalDeclarations(KDevelop::Identifier(key.second), CursorInRevision::invalid(), 0, key.first.abstractType(), KDevelop::DUContext::OnlyFunctions).isEmpty())
         overridable.insert(key, KDevelop::CompletionTreeItemPointer(new ImplementationHelperItem(ImplementationHelperItem::Override, DeclarationPointer(decl), completionContext, (classFun && classFun->isAbstract()) ? 1 : 2)));
     }
   }
@@ -1276,7 +1281,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
           {
             ifDebug( kDebug() << "functionCallAccess" << functions().count() << m_expression; )
             
-            uint max = MoreArgumentHintsCompletionItem::resetMaxArgumentHints();
+            uint max = MoreArgumentHintsCompletionItem::resetMaxArgumentHints(!fullCompletion);
             
             //Don't show annoying empty argument-hints
 /*            if(parentContext->m_contextType != BinaryOperatorFunctionCall && parentContext->functions().size() == 0)
@@ -1287,9 +1292,19 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
               int num = 0;
               foreach( const Cpp::CodeCompletionContext::Function &function, functions() ) {
                 if (num == max) {
-                  //When there are too many overloaded functions, do not show them all if completion was invoked automatically
-                  CompletionTreeItemPointer item( new MoreArgumentHintsCompletionItem( KSharedPtr <KDevelop::CodeCompletionContext >(this), i18ncp("Here, overload is used as a programming term.  This string is used to display how many overloaded versions there are of the function whose name is the second argument.", "1 more overload of %2 (show more)", "%1 more overloads of %2 (show more)", functions().count() - num, functionName()), num ) );
-                  items << item;
+                  if(fullCompletion)
+                  {
+                    //When there are too many overloaded functions, do not show them all
+                    CompletionTreeItemPointer item( new MoreArgumentHintsCompletionItem( KSharedPtr <KDevelop::CodeCompletionContext >(this), i18ncp("Here, overload is used as a programming term.  This string is used to display how many overloaded versions there are of the function whose name is the second argument.", "1 more overload of %2 (show more)", "%1 more overloads of %2 (show more)", functions().count() - num, functionName()), num ) );
+                    items.push_front(item);
+                  }else if(!items.isEmpty()) {
+                    NormalDeclarationCompletionItem* last = dynamic_cast<NormalDeclarationCompletionItem*>(items.back().data());
+                    if(last->declaration())
+                    {
+//                       last->alternativeText = i18n("(%1 more) ", functions().count() - num) + last->declaration()->identifier().toString();
+//                       last->useAlternativeText = true;
+                    }
+                  }
                   break;
                 }
 
@@ -1403,7 +1418,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
                Declaration* signalContainer = signalContainerType->declaration(m_duContext->topContext());
               if(signalContainer && signalContainer->internalContext()) {
                 IndexedString signature(m_connectedSignalNormalizedSignature);
-                foreach(const DeclarationDepthPair &decl, signalContainer->internalContext()->allDeclarations( SimpleCursor::invalid(), m_duContext->topContext(), false )) {
+                foreach(const DeclarationDepthPair &decl, signalContainer->internalContext()->allDeclarations( CursorInRevision::invalid(), m_duContext->topContext(), false )) {
                   if(decl.first->identifier() == m_connectedSignalIdentifier) {
                     if(QtFunctionDeclaration* classFun = dynamic_cast<QtFunctionDeclaration*>(decl.first)) {
                       if(classFun->isSignal() && classFun->normalizedSignature() == signature) {
@@ -1436,7 +1451,7 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
                 signalSlots << CompletionTreeItemPointer(new ImplementationHelperItem(ImplementationHelperItem::CreateSignalSlot, DeclarationPointer(connectedSignal.data()), CodeCompletionContext::Ptr(this)));
               }
               
-              foreach(const DeclarationDepthPair &candidate, decl->internalContext()->allDeclarations(SimpleCursor::invalid(), m_duContext->topContext(), false) ) {
+              foreach(const DeclarationDepthPair &candidate, decl->internalContext()->allDeclarations(CursorInRevision::invalid(), m_duContext->topContext(), false) ) {
                 if(QtFunctionDeclaration* classFun = dynamic_cast<QtFunctionDeclaration*>(candidate.first)) {
                   if((classFun->isSignal() && m_onlyShow != ShowSlots) || (memberAccessOperation() == SlotAccess && classFun->isSlot() && filterDeclaration(classFun))) {
                     NormalDeclarationCompletionItem* item = new NormalDeclarationCompletionItem( DeclarationPointer(candidate.first), KDevelop::CodeCompletionContext::Ptr(this), candidate.second );
@@ -1479,7 +1494,10 @@ QList<CompletionTreeItemPointer> CodeCompletionContext::completionItems(bool& sh
       }
     }
 
-    if(!ignoreParentContext && fullCompletion && m_parentContext && (!noMultipleBinaryOperators || m_contextType != BinaryOperatorFunctionCall || parentContext()->m_contextType != BinaryOperatorFunctionCall))
+    if(!ignoreParentContext 
+      && (fullCompletion || (Cpp::useArgumentHintInAutomaticCompletion() && depth() == 0))
+      && m_parentContext
+      && (!noMultipleBinaryOperators || m_contextType != BinaryOperatorFunctionCall || parentContext()->m_contextType != BinaryOperatorFunctionCall))
       items = parentContext()->completionItems( shouldAbort, fullCompletion ) + items;
 
     if(depth() == 0) {
