@@ -55,7 +55,7 @@ KDEVCPPDUCHAIN_EXPORT  QList<KDevelop::Declaration*> findLocalDeclarations( KDev
     return ret;
   }
   
-  ret += context->findLocalDeclarations( identifier, SimpleCursor::invalid(), topContext );
+  ret += context->findLocalDeclarations( identifier, CursorInRevision::invalid(), topContext );
   if( !ret.isEmpty() )
     return ret;
 
@@ -166,7 +166,7 @@ QList< QPair<Declaration*, int> > hideOverloadedDeclarations( const QList< QPair
   return ret;
 }
 
-QList<KDevelop::Declaration*> findDeclarationsSameLevel(KDevelop::DUContext* context, const Identifier& identifier, const KDevelop::SimpleCursor& position)
+QList<KDevelop::Declaration*> findDeclarationsSameLevel(KDevelop::DUContext* context, const Identifier& identifier, const KDevelop::CursorInRevision& position)
 {
   if( context->type() == DUContext::Namespace || context->type() == DUContext::Global ) {
     ///May have been forward-declared anywhere
@@ -381,9 +381,6 @@ KDevelop::DUContext* logicalParentContext(KDevelop::DUContext* context, KDevelop
  * Preprocess the given string using the macros from given EnvironmentFile up to the given line
  * If line is -1, all macros are respected.
  * This is a quite slow operation, because thousands of macros need to be shuffled around.
- * 
- * @todo maybe implement a version of rpp::Environment that directly works on EnvironmentFile,
- * without needing to copy all macros.
  * */
 QString preprocess( const QString& text, Cpp::EnvironmentFile* file, int line, QSet<IndexedString> disableMacros ) {
 
@@ -805,7 +802,7 @@ QualifiedIdentifier stripPrefixes(DUContext* ctx, QualifiedIdentifier id)
   if(imports.contains(id))
     return QualifiedIdentifier(); ///The id is a namespace that is imported into the current context
   
-  QList< Declaration* > basicDecls = ctx->findDeclarations(id, SimpleCursor::invalid(), AbstractType::Ptr(), 0, (DUContext::SearchFlags)(DUContext::NoSelfLookUp | DUContext::NoFiltering));
+  QList< Declaration* > basicDecls = ctx->findDeclarations(id, CursorInRevision::invalid(), AbstractType::Ptr(), 0, (DUContext::SearchFlags)(DUContext::NoSelfLookUp | DUContext::NoFiltering));
   
   if(basicDecls.isEmpty())
     return id;
@@ -813,7 +810,7 @@ QualifiedIdentifier stripPrefixes(DUContext* ctx, QualifiedIdentifier id)
   while(!id.isEmpty())
   {
     QualifiedIdentifier newId = id.mid(1);
-    QList< Declaration* > foundDecls = ctx->findDeclarations(newId, SimpleCursor::invalid(), AbstractType::Ptr(), 0, (DUContext::SearchFlags)(DUContext::NoSelfLookUp | DUContext::NoFiltering));
+    QList< Declaration* > foundDecls = ctx->findDeclarations(newId, CursorInRevision::invalid(), AbstractType::Ptr(), 0, (DUContext::SearchFlags)(DUContext::NoSelfLookUp | DUContext::NoFiltering));
     
     if(foundDecls == basicDecls)
       id = newId;
