@@ -36,9 +36,6 @@
 #include <interfaces/ilanguage.h>
 #include <language/interfaces/iastcontainer.h>
 
-#include <ktexteditor/document.h>
-#include <ktexteditor/smartinterface.h>
-
 #include "cpplanguagesupport.h"
 #include "cpphighlighting.h"
 
@@ -66,7 +63,6 @@
 #include <interfaces/iuicontroller.h>
 #include <cpppreprocessenvironment.h>
 
-//#define DUMP_SMART_RANGES
 //#define DUMP_AST
 //#define DUMP_DUCHAIN
 
@@ -115,7 +111,6 @@ CPPParseJob::CPPParseJob( const KUrl &url,
         m_needUpdateEverything( false ),
         m_parentPreprocessor( parentPreprocessor ),
         m_session( new ParseSession ),
-        m_readFromDisk( false ),
         m_includePathsComputed( 0 ),
         m_keepDuchain( false ),
         m_parsedIncludes( 0 ),
@@ -408,19 +403,17 @@ void CPPInternalParseJob::initialize() {
 
 void CPPInternalParseJob::highlightIfNeeded()
 {
-    if(!KDevelop::EditorIntegrator::documentForUrl(parentJob()->document()))
+    if(!ICore::self()->languageController()->backgroundParser()->trackerForUrl(parentJob()->document()))
       return;
 
     DUChainReadLocker l(DUChain::lock());
     ReferencedTopDUContext standardContext = DUChainUtils::standardContextForUrl(parentJob()->document().toUrl());
 
-    if(standardContext && standardContext->smartRange()) {
-      kDebug( 9007 ) << "Highlighting" << parentJob()->document().str();
-      //If the document has a smart-range, at least re-do the highlighting
-      l.unlock();
-      if ( parentJob()->cpp() && parentJob()->cpp()->codeHighlighting() )
-        parentJob()->cpp()->codeHighlighting()->highlightDUChain( standardContext.data() );
-    }
+    kDebug( 9007 ) << "Highlighting" << parentJob()->document().str();
+    //If the document has a smart-range, at least re-do the highlighting
+    l.unlock();
+    if ( parentJob()->cpp() && parentJob()->cpp()->codeHighlighting() )
+      parentJob()->cpp()->codeHighlighting()->highlightDUChain( standardContext.data() );
 }
 
 void CPPInternalParseJob::run()
@@ -428,11 +421,20 @@ void CPPInternalParseJob::run()
     //Happens during shutdown
     if(!ICore::self()->languageController()->language("C++")->languageSupport() || !parentJob()->cpp())
       return;
+    
     //If we have a parent, the parse-mutex is already locked
     QReadLocker lock(parentJob()->parentPreprocessor() ? 0 : parentJob()->cpp()->language()->parseLock());
     if(!ICore::self()->languageController()->language("C++")->languageSupport() || !parentJob()->cpp())
       return;
+    
+    initialize();
 
+    if(updatingContentContext)
+      parentJob()->translateDUChainToRevision(updatingContentContext.data());
+    
+    if(updatingProxyContext)
+      parentJob()->translateDUChainToRevision(updatingProxyContext.data());
+    
     UrlParseLock urlLock(parentJob()->document());
 
     if(!parentJob()->needsUpdate()) {
@@ -455,8 +457,6 @@ void CPPInternalParseJob::run()
 
     parentJob()->setLocalProgress(0, i18n("Parsing actual file"));
 
-    initialize();
-
     Cpp::EnvironmentFilePointer proxyEnvironmentFile(parentJob()->proxyEnvironmentFile());
     Cpp::EnvironmentFilePointer contentEnvironmentFile(parentJob()->contentEnvironmentFile());
 
@@ -531,6 +531,7 @@ void CPPInternalParseJob::run()
     ///Now we build/update the content-context
 
     bool doNotChangeDUChain = false;
+    bool isOpenInEditor = ICore::self()->languageController()->backgroundParser()->trackerForUrl(parentJob()->document());
 
     if(!parentJob()->keepDuchain()) {
 
@@ -543,7 +544,7 @@ void CPPInternalParseJob::run()
 
       ///At some point, we have to give up on features again, else processing will be just too slow.
       ///Simple solution for now: Always go down to the minimum required level.
-      if(!contentContext || !contentContext->smartRange())
+      if(!contentContext || !isOpenInEditor)
         newFeatures = parentJob()->minimumFeatures();
 
       bool keepAST = newFeatures & TopDUContext::AST;
@@ -566,8 +567,8 @@ void CPPInternalParseJob::run()
           DUChainWriteLocker l(DUChain::lock());
 
           if((updatingContentContext->features() & parentJob()->minimumFeatures()) ==  parentJob()->minimumFeatures() &&
-              updatingContentContext->smartRange() &&
-              updatingContentContext->parsingEnvironmentFile()->modificationRevision().modificationTime == ModificationRevision::revisionForFile(updatingContentContext->url()).modificationTime) {
+            isOpenInEditor &&
+              updatingContentContext->parsingEnvironmentFile()->modificationRevision().modificationTime == ModificationRevision::revisionForFile(updatingContentContext->url()).modificationTime && CppLanguageSupport::self()->codeHighlighting()->hasHighlighting(parentJob()->document())) {
             kDebug() << "not processing" << updatingContentContext->url().str() << "because of missing compound tokens";
             ICore::self()->uiController()->showErrorMessage(i18n("Not updating duchain for %1", parentJob()->document().toUrl().fileName()), 1);
             l.unlock();
@@ -591,38 +592,14 @@ void CPPInternalParseJob::run()
         }
       }
 
-      CppEditorIntegrator editor(parentJob()->parseSession().data());
       bool isStandardContext = false;
       {
         DUChainWriteLocker l(DUChain::lock());
         TopDUContext* knownStandardContext = DUChainUtils::standardContextForUrl(parentJob()->document().toUrl());
 
         isStandardContext = (parentJob()->masterJob() == parentJob() || knownStandardContext == updatingContentContext || !knownStandardContext);
-
-        if(isStandardContext) {
-          //Delete the smart-ranges of all other contexts, so we never get problems with smart uses/highlighting
-          foreach(TopDUContext* context, DUChain::self()->chainsForDocument(parentJob()->document())) {
-            if(context != updatingContentContext && context->smartRange()) {
-              SmartConverter sc(&editor);
-              sc.deconvertDUChain(context);
-              kDebug() << "smart-deconverting a non-standard context";
-            }
-          }
-
-          editor.setCurrentUrl(parentJob()->document(), true);
-
-          if(updatingContentContext && !updatingContentContext->smartRange() && editor.smart()) {
-            kWarning() << "updated context has not been smartened yet! Smartening it";
-            SmartConverter sc(&editor);
-            sc.convertDUChain(updatingContentContext);
-            Q_ASSERT(updatingContentContext->smartRange());
-            ICore::self()->languageController()->backgroundParser()->addManagedTopRange(updatingContentContext->smartRange()->document()->url(), updatingContentContext->smartRange());
-          }
-        }
       }
 
-      editor.setCurrentUrl(parentJob()->document(), isStandardContext);
-
       kDebug( 9007 ) << (contentContext ? "updating" : "building") << "duchain for" << parentJob()->document().str();
 
       uint oldItemCount = 0;
@@ -632,7 +609,7 @@ void CPPInternalParseJob::run()
         oldItemCount = contentContext->childContexts().size() + contentContext->localDeclarations().size();
       }
 
-      DeclarationBuilder declarationBuilder(&editor);
+      DeclarationBuilder declarationBuilder(parentJob()->parseSession().data());
 
       if(newFeatures == TopDUContext::VisibleDeclarationsAndContexts) {
         declarationBuilder.setOnlyComputeVisible(true); //Only visible declarations/contexts need to be built.
@@ -733,7 +710,7 @@ void CPPInternalParseJob::run()
           if ((newFeatures & TopDUContext::AllDeclarationsContextsAndUses) == TopDUContext::AllDeclarationsContextsAndUses) {
               parentJob()->setLocalProgress(0.5, i18n("Building uses"));
 
-              UseBuilder useBuilder(&editor);
+              UseBuilder useBuilder(parentJob()->parseSession().data());
               useBuilder.setMapAst(keepAST);
               useBuilder.buildUses(ast);
               DUChainWriteLocker l(DUChain::lock());
@@ -744,16 +721,16 @@ void CPPInternalParseJob::run()
               DUChainWriteLocker lock( DUChain::lock() );
               contentContext->deleteUsesRecursively();
           }
-          
-          if (!parentJob()->abortRequested() && editor.smart()) {
-
-            if ( parentJob()->cpp() && parentJob()->cpp()->codeHighlighting() )
-            {
-                parentJob()->cpp()->codeHighlighting()->highlightDUChain( contentContext );
-            }
-          }
         }
       }
+      
+      if (!parentJob()->abortRequested() && isOpenInEditor) {
+        if ( parentJob()->cpp() && parentJob()->cpp()->codeHighlighting() )
+        {
+          parentJob()->cpp()->codeHighlighting()->highlightDUChain( contentContext );
+        }
+      }
+      
 
       ///Now mark the context as not being updated. This MUST be done or we will be waiting forever in a loop
       {
@@ -795,7 +772,7 @@ void CPPInternalParseJob::run()
       ///Add all our imports to the re-used context, just to make sure they are there.
       foreach( const LineContextPair& import, importedContentChains )
           if(!import.temporary)
-            contentContext->addImportedParentContext(import.context, SimpleCursor(import.sourceLine, 0));
+            contentContext->addImportedParentContext(import.context, CursorInRevision(import.sourceLine, 0));
       contentContext->updateImportsCache();
     }
 
@@ -808,7 +785,7 @@ void CPPInternalParseJob::run()
         if(!context.context)
           continue;
         Q_ASSERT(context.context);
-        if(!contentContext->imports(context.context.data(), SimpleCursor::invalid())) {
+        if(!contentContext->imports(context.context.data(), CursorInRevision::invalid())) {
           kWarning() << "Context should be imported, but is not:" << contentContext->url().str() << " <- " << context.context->url().str();
         }
       }
@@ -834,11 +811,11 @@ void CPPInternalParseJob::run()
           Q_ASSERT(0); //Failure
         }
 
-        Q_ASSERT(proxyContext->importedParentContexts()[0].context(0) == contentContext);
+        Q_ASSERT(contentContextFromProxyContext(proxyContext) == contentContext.data());
 
         //Make sure the imported contextsa re added
         foreach ( const LineContextPair &context, parentJob()->includedFiles() )
-          proxyContext->addImportedParentContext(context.context, SimpleCursor(context.sourceLine, 0));
+          proxyContext->addImportedParentContext(context.context, CursorInRevision(context.sourceLine, 0));
 
         proxyContext->updateImportsCache();
 
@@ -846,7 +823,7 @@ void CPPInternalParseJob::run()
 
         //Put the problems into the proxy-context
         foreach( const ProblemPointer& problem, parentJob()->preprocessorProblems() ) {
-          if(problem->finalLocation().start().line() < proxyEnvironmentFile->contentStartLine())
+          if(problem->finalLocation().start.line < proxyEnvironmentFile->contentStartLine())
             proxyContext->addProblem(problem);
         }
 
@@ -890,44 +867,16 @@ void CPPInternalParseJob::run()
             DUChainReadLocker lock(DUChain::lock());
 #ifdef DUMP_DUCHAIN
         kDebug( 9007 ) << "================== duchain ==================";
-        KDevelop::DumpChain dump;
-        dump.dump(contentContext);
-#endif
-#ifdef DUMP_SMART_RANGES
-        if(contentContext->smartRange()) {
-          kDebug() << "dumping smart range";
-          KDevelop::DumpChain dump;
-          kDebug() << dump.dumpRanges(contentContext->smartRange());
-        }
-
+        KDevelop::dumpDUContext(contentContext);
 #endif
 
         //KDevelop::DumpDotGraph dumpGraph;
         //kDebug(9007) << "Dot-graph:\n" << dumpGraph.dotGraph(topContext, true);
     }
 
-    //DumpTree dumpTree;
-
     kDebug( 9007 ) << "===-- Parsing finished --===>" << parentJob()->document().str();
 
     parentJob()->processDelayedImports();
-
-    //Check the import structure
-/*    if(parentJob()->masterJob() == parentJob()) {
-      DUChainReadLocker l(DUChain::lock());
-      foreach(const DUContext* context, parentJob()->updated()) {
-        if(context == contentContext || context == proxyContext)
-          continue;
-        if(static_cast<const TopDUContext*>(context)->flags() & TopDUContext::ProxyContextFlag){
-//           Q_ASSERT(proxyContext->imports(context, proxyContext->range().end));
-        }else{
-          kDebug( 9007 ) << "lost an instance of" << context->url().str() << "with" << context->childContexts().count() << "child-contexts and " << context->localDeclarations().count() << "local declarations";
-//          Q_ASSERT(contentContext->imports(context, contentContext->range().end));
-        }
-      }
-    }*/
-
-   parentJob()->cleanupSmartRevision();
 }
 
 void CPPParseJob::processDelayedImports() {
@@ -942,7 +891,7 @@ void CPPParseJob::processDelayedImports() {
       Q_ASSERT(context.context->parsingEnvironmentFile());
       if(context.context->parsingEnvironmentFile()->isProxyContext()) {
         Q_ASSERT(m_parseJob->proxyContext);
-        context.context->addImportedParentContext(m_parseJob->proxyContext.data(), SimpleCursor(context.sourceLine, 0));
+        context.context->addImportedParentContext(m_parseJob->proxyContext.data(), CursorInRevision(context.sourceLine, 0));
         Cpp::EnvironmentFile* cppEnvFile = dynamic_cast<Cpp::EnvironmentFile*>(context.context->parsingEnvironmentFile().data());
         Q_ASSERT(cppEnvFile);
         cppEnvFile->merge(dynamic_cast<Cpp::EnvironmentFile&>(*m_parseJob->proxyContext->parsingEnvironmentFile().data()));
@@ -953,7 +902,7 @@ void CPPParseJob::processDelayedImports() {
       if(!content.context)
         continue;
       Q_ASSERT(content.context);
-      content.context->addImportedParentContext(m_parseJob->proxyContext.data(), SimpleCursor(content.sourceLine, 0));
+      content.context->addImportedParentContext(m_parseJob->proxyContext.data(), CursorInRevision(content.sourceLine, 0));
       content.context->updateImportsCache();
       Cpp::EnvironmentFile* cppEnvFile = dynamic_cast<Cpp::EnvironmentFile*>(content.context->parsingEnvironmentFile().data());
       Q_ASSERT(cppEnvFile);
@@ -979,16 +928,6 @@ ParseSession::Ptr CPPParseJob::parseSession() const
     return m_session;
 }
 
-void CPPParseJob::setReadFromDisk(bool readFromDisk)
-{
-    m_readFromDisk = readFromDisk;
-}
-
-bool CPPParseJob::wasReadFromDisk() const
-{
-    return m_readFromDisk;
-}
-
 CPPInternalParseJob * CPPParseJob::parseJob() const
 {
     return m_parseJob;
