@@ -58,7 +58,6 @@
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/duchainutils.h>
 #include <language/codecompletion/codecompletion.h>
-#include <language/codegen/applychangeswidget.h>
 
 #include "cmakemodelitems.h"
 #include "cmakenavigationwidget.h"
@@ -77,10 +76,7 @@
 #endif
 
 #include "ui_cmakepossibleroots.h"
-#include <language/editor/editorintegrator.h>
-#include <language/duchain/smartconverter.h>
 #include <language/duchain/use.h>
-#include <ktexteditor/smartinterface.h>
 #include <interfaces/idocumentation.h>
 #include "cmakeprojectdata.h"
 #include <cmakeconfig.h>
@@ -168,30 +164,34 @@ CMakeManager::~CMakeManager()
 KUrl CMakeManager::buildDirectory(KDevelop::ProjectBaseItem *item) const
 {
     KUrl ret;
-    if(item) {
-        bool isroot = false;
-        if (ProjectFolderItem* projectFolderItem = item->folder())
-            isroot = !projectFolderItem->parent();
-        
-        if (isroot) {
-            ret=CMake::currentBuildDir(item->project());
-        } else {
-            ret=buildDirectory(item->parent());
-            CMakeFolderItem *fi=dynamic_cast<CMakeFolderItem*>(item);
-            if(fi)
-                ret.addPath(fi->buildDir());
-        }
-    }
+    ProjectBaseItem* parent = item->parent();
+    if (parent)
+        ret=buildDirectory(parent);
+    else
+        ret=CMake::currentBuildDir(item->project());
+    
+    CMakeFolderItem *fi=dynamic_cast<CMakeFolderItem*>(item);
+    if(fi)
+        ret.addPath(fi->buildDir());
     return ret;
 }
 
-KDevelop::ReferencedTopDUContext CMakeManager::initializeProject(KDevelop::IProject* project, const KUrl& baseUrl)
+KDevelop::ReferencedTopDUContext CMakeManager::initializeProject(KDevelop::IProject* project)
 {
+    KUrl baseUrl=project->folder();
+    baseUrl.cd(CMake::projectRootRelative(project));
+    
     QPair<VariableMap,QStringList> initials = CMakeParserUtils::initialVariables();
-    m_modulePathPerProject[project]=initials.first["CMAKE_MODULE_PATH"];
-    m_macrosPerProject[project].clear();
-    m_varsPerProject[project]=initials.first;
-    m_varsPerProject[project].insert("CMAKE_SOURCE_DIR", QStringList(baseUrl.toLocalFile(KUrl::RemoveTrailingSlash)));
+    CMakeProjectData* data = &m_projectsData[project];
+    
+    data->clear();
+    data->modulePath=initials.first["CMAKE_MODULE_PATH"];
+    data->vm=initials.first;
+    data->vm.insert("CMAKE_SOURCE_DIR", QStringList(baseUrl.toLocalFile(KUrl::RemoveTrailingSlash)));
+    
+    KUrl cachefile=buildDirectory(project->projectItem());
+    cachefile.addPath("CMakeCache.txt");
+    data->cache=readCache(cachefile);
 
     KSharedConfig::Ptr cfg = project->projectConfiguration();
     KConfigGroup group(cfg.data(), "CMake");
@@ -202,17 +202,17 @@ KDevelop::ReferencedTopDUContext CMakeManager::initializeProject(KDevelop::IProj
         {
             if( QFileInfo(path).exists() )
             {
-                m_modulePathPerProject[project] << path;
+                data->modulePath << path;
                 l << path;
             }
         }
         if( !l.isEmpty() )
             group.writeEntry("CMakeDir", l);
         else
-            group.writeEntry("CMakeDir", m_modulePathPerProject[project]);
+            group.writeEntry("CMakeDir", data->modulePath);
     }
     else
-        group.writeEntry("CMakeDir", m_modulePathPerProject[project]);
+        group.writeEntry("CMakeDir", data->modulePath);
 
     
     KDevelop::ReferencedTopDUContext buildstrapContext;
@@ -229,7 +229,7 @@ KDevelop::ReferencedTopDUContext CMakeManager::initializeProject(KDevelop::IProj
             buildstrapContext->deleteChildContextsRecursively();
         }else{
             IndexedString idxpath(buildStrapUrl);
-            buildstrapContext=new TopDUContext(idxpath, SimpleRange(0,0, 0,0),
+            buildstrapContext=new TopDUContext(idxpath, RangeInRevision(0,0, 0,0),
                                                new ParsingEnvironmentFile(idxpath));
             DUChain::self()->addDocumentChain(buildstrapContext);
         }
@@ -239,10 +239,32 @@ KDevelop::ReferencedTopDUContext CMakeManager::initializeProject(KDevelop::IProj
     ReferencedTopDUContext ref=buildstrapContext;
     foreach(const QString& script, initials.second)
     {
-        ref = includeScript(CMakeProjectVisitor::findFile(script, m_modulePathPerProject[project], QStringList()),
-                              project, ref);
+        ref = includeScript(CMakeProjectVisitor::findFile(script, m_projectsData[project].modulePath, QStringList()), project, baseUrl.toLocalFile(), ref);
     }
     
+    //Initialize parent parts of the project that don't belong to the tree (because it's a partial import)
+    if(baseUrl.isParentOf(project->folder()) && baseUrl!=project->folder())
+    {
+        QList<KUrl> toimport;
+        toimport += baseUrl;
+        while(!toimport.isEmpty()) {
+            KUrl script = toimport.takeFirst(), currentDir=script;
+            script.addPath("CMakeLists.txt");
+            
+            ref = includeScript(script.toLocalFile(), project, currentDir.toLocalFile(), ref);
+            Q_ASSERT(ref);
+            
+            foreach(const Subdirectory& s, data->subdirectories) {
+                KUrl candidate = currentDir;
+                candidate.addPath(s.name);
+                
+                if(candidate.isParentOf(project->folder()))
+                    toimport += candidate;
+            }
+        }
+        
+        dynamic_cast<CMakeFolderItem*>(project->projectItem())->setBuildDir(KUrl::relativeUrl(baseUrl, project->folder()));
+    }
     return ref;
 }
 
@@ -299,14 +321,13 @@ KDevelop::ProjectFolderItem* CMakeManager::import( KDevelop::IProject *project )
             }
         }
 
-        m_rootItem = new CMakeFolderItem(project, folderUrl, QString(), 0 );
+        m_rootItem = new CMakeFolderItem(project, project->folder(), QString(), 0 );
 
         KUrl cachefile=buildDirectory(m_rootItem);
         if( cachefile.isEmpty() ) {
             CMake::checkForNeedingConfigure(m_rootItem);
         }
         cachefile.addPath("CMakeCache.txt");
-        m_projectCache[project]=readCache(cachefile);
         
         KDirWatch* w = new KDirWatch(project);
         w->setObjectName(project->name()+"_ProjectWatcher");
@@ -321,24 +342,24 @@ KDevelop::ProjectFolderItem* CMakeManager::import( KDevelop::IProject *project )
 
 
 KDevelop::ReferencedTopDUContext CMakeManager::includeScript(const QString& file,
-                                                        KDevelop::IProject * project, ReferencedTopDUContext parent)
+                                                        KDevelop::IProject * project, const QString& dir, ReferencedTopDUContext parent)
 {
     m_watchers[project]->addFile(file);
-    return CMakeParserUtils::includeScript( file, parent, &m_varsPerProject[project], &m_macrosPerProject[project], project->folder().toLocalFile(KUrl::RemoveTrailingSlash), &m_projectCache[project], m_modulePathPerProject[project]);
+    return CMakeParserUtils::includeScript( file, parent, &m_projectsData[project], dir);
 }
 
-QMutex rxFileFilterMutex; //We have to use a mutex-lock to protect static regular ex
-static QRegExp rxFileFilter("\\w*~$|\\w*\\.bak$"); ///@todo This filter should be configurable, and filtering should be done on a manager-independent level
+
 
 QSet<QString> filterFiles(const QStringList& orig)
 {
-    QMutexLocker lock(&rxFileFilterMutex);
-    
     QSet<QString> ret;
     foreach(const QString& str, orig)
     {
-        if(rxFileFilter.indexIn(str)<0)
-            ret.insert(str);
+        ///@todo This filter should be configurable, and filtering should be done on a manager-independent level
+        if (str.endsWith(QLatin1Char('~')) || str.endsWith(QLatin1String(".bak")))
+            continue;
+
+        ret.insert(str);
     }
     return ret;
 }
@@ -348,7 +369,6 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
     QList<KDevelop::ProjectFolderItem*> folderList;
     CMakeFolderItem* folder = dynamic_cast<CMakeFolderItem*>( item );
 
-    KUrl subroot=item->project()->folder();
     m_watchers[item->project()]->addDir(item->url().toLocalFile(), KDirWatch::WatchFiles);
     
     KUrl cmakeListsPath(folder->url());
@@ -360,72 +380,28 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
         
         KDevelop::ReferencedTopDUContext curr;
         if(item==item->project()->projectItem())
-            curr=initializeProject(item->project(), item->project()->projectItem()->url());
+            curr=initializeProject(item->project());
         else
             curr=folder->formerParent()->topDUContext();
         
         kDebug(9042) << "Adding cmake: " << cmakeListsPath << " to the model";
 
-        m_watchers[item->project()]->addFile(cmakeListsPath.toLocalFile());
         QString binDir=KUrl::relativePath(folder->project()->projectItem()->url().toLocalFile(), folder->url().toLocalFile());
         if(binDir.startsWith("./"))
             binDir=binDir.remove(0, 2);
         
-        CMakeProjectData data;
-        
-        //Im not sure if we want to save taht, it might be a lot of data,
-        //but can be useful when regenerating.
-        data.vm=m_varsPerProject[item->project()];
-        data.mm=m_macrosPerProject[item->project()];
-        QString currentBinDir=data.vm.value("CMAKE_BINARY_DIR")[0]+binDir;
+        CMakeProjectData& data=m_projectsData[item->project()];
 
-        data.vm.insert("CMAKE_CURRENT_BINARY_DIR", QStringList(currentBinDir));
-        data.vm.insert("CMAKE_CURRENT_LIST_FILE", QStringList(cmakeListsPath.toLocalFile(KUrl::RemoveTrailingSlash)));
-        data.vm.insert("CMAKE_CURRENT_LIST_DIR", QStringList(folder->url().toLocalFile(KUrl::RemoveTrailingSlash)));
-        data.vm.insert("CMAKE_CURRENT_SOURCE_DIR", QStringList(folder->url().toLocalFile(KUrl::RemoveTrailingSlash)));
-
-        kDebug(9042) << "currentBinDir" << KUrl(data.vm.value("CMAKE_BINARY_DIR")[0]) << data.vm.value("CMAKE_CURRENT_BINARY_DIR");
+//         kDebug(9042) << "currentBinDir" << KUrl(data.vm.value("CMAKE_BINARY_DIR")[0]) << data.vm.value("CMAKE_CURRENT_BINARY_DIR");
 
     #ifdef CMAKEDEBUGVISITOR
         CMakeAstDebugVisitor dv;
         dv.walk(cmakeListsPath.toLocalFile(), f, 0);
     #endif
-        
-        {
-            CMakeProjectVisitor v(folder->url().toLocalFile(KUrl::RemoveTrailingSlash), curr);
-            v.setCacheValues(&m_projectCache[item->project()]);
-            v.setVariableMap(&data.vm);
-            v.setMacroMap(&data.mm);
-            v.setModulePath(m_modulePathPerProject[item->project()]);
-            v.setDefinitions(folder->definitions());
-            
-            CMakeFileContent f = CMakeListsParser::readCMakeFile(cmakeListsPath.toLocalFile());
-            if(!f.isEmpty())
-                v.walk(f, 0);
-            
-            folder->setTopDUContext(v.context());
-            data.projectName=v.projectName();
-            data.subdirectories=v.subdirectories();
-            data.definitions=v.definitions();
-            data.includeDirectories=v.includeDirectories();
-            data.targets=v.targets();
-            data.properties=v.properties();
-            
-            QList<Target>::iterator it=data.targets.begin(), itEnd=data.targets.end();
-            for(; it!=itEnd; ++it)
-            {
-                it->files=v.resolveDependencies(it->files);
-            }
-        }
-        data.vm.remove("CMAKE_CURRENT_LIST_FILE");
-        data.vm.remove("CMAKE_CURRENT_LIST_DIR");
-        data.vm.remove("CMAKE_CURRENT_SOURCE_DIR");
-        data.vm.remove("CMAKE_CURRENT_BINARY_DIR");
-        
-        m_varsPerProject[item->project()]=data.vm;
-        m_macrosPerProject[item->project()]=data.mm;
 
-        /*{
+        ReferencedTopDUContext ctx = includeScript(cmakeListsPath.toLocalFile(), folder->project(), item->url().toLocalFile(), curr);
+        folder->setTopDUContext(ctx);
+       /*{
         kDebug() << "dumpiiiiiing" << folder->url();
         KDevelop::DUChainReadLocker lock(KDevelop::DUChain::lock());
         KDevelop::dumpDUContext(v.context(), false);
@@ -445,9 +421,7 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
                 path.addPath(subf.name);
             }
             path.adjustPath(KUrl::AddTrailingSlash);
-
-            kDebug(9042) << "Found subdir " << path << "which should be into" << subroot;
-            if((subroot.isParentOf(path) || path.isParentOf(subroot)) && QFileInfo(path.toLocalFile()).isDir())
+            
             {
                 alreadyAdded.append(subf.name);
                 CMakeFolderItem* parent=folder;
@@ -463,12 +437,12 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
                         a = static_cast<CMakeFolderItem*>(ff);
                     
                 }
-                if(!a) {
+                if(!a)
                     a = new CMakeFolderItem( folder->project(), path, subf.build_dir, parent );
-                }
+                else
+                    a->setUrl(path);
                 
-                kDebug() << "folder: " << a << a->index();
-                a->setUrl(path);
+//                 kDebug() << "folder: " << a << a->index();
                 a->setDefinitions(data.definitions);
                 folderList.append( a );
                 
@@ -487,7 +461,6 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
 
         QStringList directories;
         directories += folder->url().toLocalFile(KUrl::RemoveTrailingSlash);
-        directories += currentBinDir;
 
         foreach(const QString& s, data.includeDirectories)
         {
@@ -506,7 +479,6 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
                 dir=simp.toLocalFile();
             }
 
-            kDebug() << "converting " << s << dir;
             if(!directories.contains(dir))
                 directories.append(dir);
         }
@@ -568,12 +540,11 @@ QList<KDevelop::ProjectFolderItem*> CMakeManager::parse( KDevelop::ProjectFolder
                 KUrl sourceFile(sFile);
                 if(sourceFile.isRelative()) {
                     sourceFile = folder->url();
-                    sourceFile.adjustPath( KUrl::RemoveTrailingSlash );
                     sourceFile.addPath( sFile );
                 }
                 
                 tfiles += sourceFile;
-                kDebug(9042) << "..........Adding:" << sourceFile;
+                kDebug(9042) << "..........Adding:" << sourceFile << sFile << folder->url();
             }
             
             setTargetFiles(targetItem, tfiles);
@@ -822,7 +793,6 @@ void CMakeManager::dirtyFile(const QString & dirty)
         }
         
         if(p) {
-            m_projectCache[p]=readCache(dirtyFile);
             p->reloadModel();
         }
     } else if(dirty.endsWith(".cmake"))
@@ -880,18 +850,7 @@ void CMakeManager::reloadFiles(ProjectFolderItem* item)
         
         if(!entries.contains(current))
         {
-            switch(it->type())
-            {
-                case ProjectBaseItem::File:
-                    qDeleteAll(item->project()->filesForUrl(fileurl));
-                    break;
-                case ProjectBaseItem::Folder:
-                case ProjectBaseItem::BuildFolder:
-                    qDeleteAll(item->project()->foldersForUrl(fileurl));
-                    break;
-                default:
-                    break;
-            }
+            qDeleteAll(item->project()->itemsForUrl(fileurl));
         }
         else {
             if(it->url()!=fileurl) {
@@ -911,7 +870,8 @@ void CMakeManager::reloadFiles(ProjectFolderItem* item)
         fileurl.addPath( entry );
 
         // existing entries should have been removed above already
-        Q_ASSERT( !item->hasFileOrFolder( entry ) );
+        // disabled for performance reasons
+//         Q_ASSERT( !item->hasFileOrFolder( entry ) );
 
         if( QFileInfo( fileurl.toLocalFile() ).isDir() )
         {
@@ -965,7 +925,7 @@ KDevelop::ILanguage * CMakeManager::language()
     return core()->languageController()->language(name());
 }
 
-const KDevelop::ICodeHighlighting* CMakeManager::codeHighlighting() const
+KDevelop::ICodeHighlighting* CMakeManager::codeHighlighting() const
 {
     return m_highlight;
 }
@@ -1005,7 +965,7 @@ void CMakeManager::jumpToDeclaration()
             Declaration* decl = du->declaration().data();
             if(!decl)
                 return;
-            c = decl->range().start.textCursor();
+            c = decl->rangeInCurrentRevision().start.textCursor();
             url = decl->url().toUrl();
         }
 
@@ -1047,8 +1007,8 @@ CacheValues CMakeManager::readCache(const KUrl &path) const
 
 KDevelop::ProjectFolderItem* CMakeManager::addFolder( const KUrl& folder, KDevelop::ProjectFolderItem* parent)
 {
-    if(!dynamic_cast<CMakeFolderItem*>(parent)) {
-        KDevelop::createFolder(folder);
+    bool created = KDevelop::createFolder(folder);
+    if(!created || !dynamic_cast<CMakeFolderItem*>(parent)) {
         return 0;
     }
     
@@ -1070,7 +1030,7 @@ KDevelop::ProjectFolderItem* CMakeManager::addFolder( const KUrl& folder, KDevel
     if(e.exec())
     {
         bool saved=e.applyAllChanges();
-        if(saved && KDevelop::createFolder(folder)) { //If saved we create the folder then the CMakeLists.txt file
+        if(saved) { //If saved we create the folder then the CMakeLists.txt file
             KUrl newCMakeLists(folder);
             newCMakeLists.addPath("CMakeLists.txt");
 
@@ -1085,48 +1045,14 @@ KDevelop::ProjectFolderItem* CMakeManager::addFolder( const KUrl& folder, KDevel
     return 0;
 }
 
-bool CMakeManager::removeFolder( KDevelop::ProjectFolderItem* it)
-{
-    if ( !KDevelop::removeUrl(it->project(), it->url(), true) ) {
-        return false;
-    }
-    
-    if(it->type()!=KDevelop::ProjectBaseItem::BuildFolder) {
-        it->parent()->removeRow(it->row());
-        return true;
-    }
-    
-    KUrl lists=it->url().upUrl();
-    lists.addPath("CMakeLists.txt");
-
-    ApplyChangesWidget e;
-    e.setCaption(it->text());
-    e.setInformation(i18n("Remove a folder called '%1'.", it->text()));
-    e.addDocuments(IndexedString(lists));
-    
-    CMakeFolderItem* cmit=static_cast<CMakeFolderItem*>(it);
-    KTextEditor::Range r=cmit->descriptor().range().textRange();
-    kDebug(9042) << "For " << lists << " remove " << r;
-    e.document()->removeText(r);
-
-    if(e.exec())
-    {
-        bool saved=e.applyAllChanges();
-        if(!saved)
-            KMessageBox::error(0, i18n("KDevelop - CMake Support"),
-                                  i18n("Could not save the change."));
-    }
-    return 0;
-}
-
-bool followUses(KTextEditor::Document* doc, SimpleRange r, const QString& name, const KUrl& lists, bool add, const QString& replace)
+bool followUses(KTextEditor::Document* doc, RangeInRevision r, const QString& name, const KUrl& lists, bool add, const QString& replace)
 {
     bool ret=false;
-    QString txt=doc->text(r.textRange());
+    QString txt=doc->text(r.castToSimpleRange().textRange());
     if(!add && txt.contains(name))
     {
         txt.replace(name, replace);
-        doc->replaceText(r.textRange(), txt);
+        doc->replaceText(r.castToSimpleRange().textRange(), txt);
         ret=true;
     }
     else
@@ -1149,7 +1075,7 @@ bool followUses(KTextEditor::Document* doc, SimpleRange r, const QString& name,
 
         if(add && decls.isEmpty())
         {
-            doc->insertText(r.textRange().start(), name);
+            doc->insertText(r.castToSimpleRange().textRange().start(), name);
             ret=true;
         }
         else foreach(Declaration* d, decls)
@@ -1160,7 +1086,7 @@ bool followUses(KTextEditor::Document* doc, SimpleRange r, const QString& name,
             {
                 int endParenIndex = doc->line(lineNum).indexOf(')');
                 if(endParenIndex >= 0) {
-                    r.end = SimpleCursor(lineNum, endParenIndex);
+                    r.end = CursorInRevision(lineNum, endParenIndex);
                     break;
                 }
             }
@@ -1174,51 +1100,137 @@ bool followUses(KTextEditor::Document* doc, SimpleRange r, const QString& name,
     return ret;
 }
 
-bool CMakeManager::removeFile( KDevelop::ProjectFileItem* it)
+QList<TargetFilePair> CMakeManager::getTargetFilesWithin(const QList<KDevelop::ProjectBaseItem*> &items) const
 {
-    bool ret=true;
-    QList<ProjectFileItem*> files=it->project()->filesForUrl(it->url());
-    QMap<ProjectTargetItem*, ProjectFileItem*> targets;
-
-    //We loop through all the files with the same url
-    foreach(ProjectFileItem* file, files)
+    QList<TargetFilePair> targetFiles;
+    foreach (ProjectBaseItem* item, items)
     {
-        ProjectTargetItem* target=file->parent()->target();
-        if(target)
-            targets.insert(target, file);
+        if (ProjectFileItem* file = item->file())
+        {
+            //FIXME: ProjectFileItems should have a list of the targets they belong to
+            QList<ProjectFileItem*> allItemsForUrl = file->project()->filesForUrl(file->url());
+            foreach(ProjectFileItem* item, allItemsForUrl)
+            {
+                if (ProjectTargetItem* target = item->parent()->target())
+                    targetFiles << TargetFilePair(target, item);
+            }
+        }
+        else if (ProjectFolderItem* folder = item->folder())
+            targetFiles += getTargetFilesWithin(folder->children());
     }
-    
-    QMap< ProjectTargetItem*, ProjectFileItem* >::const_iterator it2 = targets.constBegin();
-    for(; it2!=targets.constEnd(); ++it2)
-        ret = ret && removeFileFromTarget(it2.value(), it2.key());
-
-    return ret && KDevelop::removeUrl(it->project(), it->url(), false);
+    return targetFiles;
 }
 
-bool CMakeManager::removeFileFromTarget( KDevelop::ProjectFileItem* it, KDevelop::ProjectTargetItem* target)
+QList<CMakeFolderItem*> CMakeManager::getCMakeFoldersWithin(const QList<KDevelop::ProjectBaseItem*> &items) const
 {
-    if(it->parent()!=target)
-        return false; //It is not a cmake-managed file
+    QList<CMakeFolderItem*> cmakeFolders;
+    foreach (ProjectBaseItem* item, items)
+    {
+        switch (item->type())
+        {
+        case KDevelop::ProjectBaseItem::BuildFolder:
+            cmakeFolders << static_cast<CMakeFolderItem*>(item);
+            //let it drop
+        case KDevelop::ProjectBaseItem::Folder:
+            getCMakeFoldersWithin(item->children());
+            break;
+        }
+    }
+    return cmakeFolders;
+}
 
-    CMakeFolderItem* folder=static_cast<CMakeFolderItem*>(target->parent());
+bool CMakeManager::changesWidgetAddCMakeFolderRemovals(const QList<CMakeFolderItem*> &folders, ApplyChangesWidget* changesWidget)
+{
+    foreach(CMakeFolderItem* folder, folders)
+    {
+        KUrl lists = folder->url().upUrl();
+        lists.addPath("CMakeLists.txt");
 
-    DescriptorAttatched* desc=dynamic_cast<DescriptorAttatched*>(target);
-    SimpleRange r=desc->descriptor().range();
-    r.start=SimpleCursor(desc->descriptor().arguments.first().range().end);
+        changesWidget->addDocuments(IndexedString(lists));
+        KTextEditor::Range range = folder->descriptor().range().castToSimpleRange().textRange();
+        if (!changesWidget->document()->removeText(range))
+            return false;
+    }
+    return true;
+}
 
-    KUrl lists=folder->url();
-    lists.addPath("CMakeLists.txt");
+bool CMakeManager::changesWidgetAddTargetFileRemovals(const QList<TargetFilePair> &targetFiles, ApplyChangesWidget* changesWidget)
+{
+    foreach (TargetFilePair targetFile, targetFiles)
+    {
+        Q_ASSERT(targetFile.second->parent() == targetFile.first);
 
-    ApplyChangesWidget e;
-    e.setCaption(it->text());
-    e.setInformation(i18n("Remove a file called '%1'.", it->text()));
-    e.addDocuments(IndexedString(lists));
+        CMakeFolderItem* folder = static_cast<CMakeFolderItem*>(targetFile.first->parent());
 
-    bool ret=followUses(e.document(), r, ' '+it->text(), lists, false, QString());
-    ret &= e.exec();
-    ret &= e.applyAllChanges();
-    
-    return ret;
+        DescriptorAttatched* desc = dynamic_cast<DescriptorAttatched*>(targetFile.first);
+        RangeInRevision range = desc->descriptor().range();
+        range.start = CursorInRevision(desc->descriptor().arguments.first().range().end);
+
+        KUrl lists = folder->url();
+        lists.addPath("CMakeLists.txt");
+
+        changesWidget->addDocuments(IndexedString(lists));
+
+        //FIXME: Not sure if this "find" extraction will cover all use cases
+        QString find = KUrl::relativeUrl(folder->url(), targetFile.second->url());
+        if (!followUses(changesWidget->document(), range, ' '+find, lists, false, QString()))
+            return false;
+    }
+    return true;
+}
+
+bool CMakeManager::removeFilesAndFolders( QList<KDevelop::ProjectBaseItem*> items)
+{
+    //First do CMakeLists changes
+    ApplyChangesWidget changesWidget;
+    changesWidget.setCaption(i18n("CMakeLists Changes"));
+    changesWidget.setInformation(i18n("Remove files and folders from CMakeLists as follows:"));
+
+    bool cmakeSuccessful = true;
+    cmakeSuccessful &= changesWidgetAddTargetFileRemovals(getTargetFilesWithin(items), &changesWidget);
+    cmakeSuccessful &= changesWidgetAddCMakeFolderRemovals(getCMakeFoldersWithin(items), &changesWidget);
+
+    if (changesWidget.hasDocuments())
+        cmakeSuccessful &= changesWidget.exec() && changesWidget.applyAllChanges();
+
+    if (!cmakeSuccessful)
+    {
+        if (KMessageBox::questionYesNo( QApplication::activeWindow(),
+                                        i18n("Changes to CMakeLists failed, abort file deletion?"),
+                                        i18n("Error"))
+            == KMessageBox::Yes)
+        {
+            return false;
+        }
+    }
+
+    //Then delete the files/folders
+    foreach(ProjectBaseItem* item, items)
+    {
+        Q_ASSERT(item->folder() || item->file());
+        Q_ASSERT(!item->file() || !item->file()->parent()->target());
+
+        if (!KDevelop::removeUrl(item->project(), item->url(), false))
+            return false;
+    }
+
+    return true;
+}
+
+bool CMakeManager::removeFilesFromTargets( QList<TargetFilePair> targetFiles)
+{
+    ApplyChangesWidget changesWidget;
+    changesWidget.setCaption(i18n("CMakeLists Changes"));
+    changesWidget.setInformation(i18n("Modify project targets as follows:"));
+
+    if (targetFiles.size() &&
+        changesWidgetAddTargetFileRemovals(targetFiles, &changesWidget) &&
+        changesWidget.exec() &&
+        changesWidget.applyAllChanges()) {
+        return true;
+    }
+
+    return false;
 }
 
 //This is being called from ::parse() so we shouldn't make it block the ui
@@ -1248,8 +1260,8 @@ bool CMakeManager::addFileToTarget( KDevelop::ProjectFileItem* it, KDevelop::Pro
     CMakeFolderItem* folder=static_cast<CMakeFolderItem*>(target->parent());
 
     DescriptorAttatched* desc=dynamic_cast<DescriptorAttatched*>(target);
-    SimpleRange r=desc->descriptor().range();
-    r.start=SimpleCursor(desc->descriptor().arguments.first().range().end);
+    RangeInRevision r=desc->descriptor().range();
+    r.start=CursorInRevision(desc->descriptor().arguments.first().range().end);
 
     KUrl lists=folder->url();
     lists.addPath("CMakeLists.txt");
@@ -1279,7 +1291,7 @@ QWidget* CMakeManager::specialLanguageObjectNavigationWidget(const KUrl& url, co
     QString htmlDoc;
     if(top)
     {
-        int useAt=top->findUseAt(position);
+        int useAt=top->findUseAt(top->transformToLocalRevision(position));
         if(useAt>=0)
         {
             Use u=top->uses()[useAt];
@@ -1322,15 +1334,15 @@ QWidget* CMakeManager::specialLanguageObjectNavigationWidget(const KUrl& url, co
 QPair<QString, QString> CMakeManager::cacheValue(KDevelop::IProject* project, const QString& id) const
 {
     QPair<QString, QString> ret;
-    if(project==0 && !m_projectCache.keys().isEmpty())
+    if(project==0 && !m_projectsData.keys().isEmpty())
     {
-        project=m_projectCache.keys().first();
+        project=m_projectsData.keys().first();
     }
     
-    kDebug() << "cache value " << id << project << (m_projectCache.contains(project) && m_projectCache[project].contains(id));
-    if(m_projectCache.contains(project) && m_projectCache[project].contains(id))
+//     kDebug() << "cache value " << id << project << (m_projectsData.contains(project) && m_projectsData[project].cache.contains(id));
+    if(m_projectsData.contains(project) && m_projectsData[project].cache.contains(id))
     {
-        const CacheEntry& e=m_projectCache[project].value(id);
+        const CacheEntry& e=m_projectsData[project].cache.value(id);
         ret.first=e.value;
         ret.second=e.doc;
     }
@@ -1369,8 +1381,8 @@ bool CMakeManager::renameFile(ProjectFileItem* it, const KUrl& newUrl)
         CMakeFolderItem* folder=static_cast<CMakeFolderItem*>(target->parent());
 
         DescriptorAttatched* desc=dynamic_cast<DescriptorAttatched*>(target);
-        SimpleRange r=desc->descriptor().range();
-        r.start=SimpleCursor(desc->descriptor().arguments.first().range().end);
+        RangeInRevision r=desc->descriptor().range();
+        r.start=CursorInRevision(desc->descriptor().arguments.first().range().end);
 
         KUrl lists=folder->url();
         lists.addPath("CMakeLists.txt");
@@ -1414,8 +1426,8 @@ bool CMakeManager::renameFolder(ProjectFolderItem* _it, const KUrl& newUrl)
     e.setCaption(it->text());
     e.setInformation(i18n("Rename a folder called '%1'.", it->text()));
     e.addDocuments(IndexedString(lists));
-    
-    KTextEditor::Range r=it->descriptor().argRange().textRange();
+
+    KTextEditor::Range r=it->descriptor().argRange().castToSimpleRange().textRange();
     kDebug(9042) << "For " << lists << " rename " << r;
     
     bool ret = e.document()->replaceText(r, newName);
@@ -1428,11 +1440,8 @@ bool CMakeManager::renameFolder(ProjectFolderItem* _it, const KUrl& newUrl)
 
 void CMakeManager::projectClosing(IProject* p)
 {
-    m_modulePathPerProject.remove(p);
-    m_varsPerProject.remove(p); 
-    m_macrosPerProject.remove(p);
+    m_projectsData.remove(p); 
     m_watchers.remove(p);
-    m_projectCache.remove(p);
 }
 
 #include "cmakemanager.moc"
